# coding: utf-8

# flake8: noqa

"""
    LUSID API

    # Introduction  This page documents the [LUSID APIs](https://www.lusid.com/api/swagger), which allows authorised clients to query and update their data within the LUSID platform.  SDKs to interact with the LUSID APIs are available in the following languages :  * [C#](https://github.com/finbourne/lusid-sdk-csharp) * [Java](https://github.com/finbourne/lusid-sdk-java) * [JavaScript](https://github.com/finbourne/lusid-sdk-js) * [Python](https://github.com/finbourne/lusid-sdk-python)  # Data Model  The LUSID API has a relatively lightweight but extremely powerful data model. One of the goals of LUSID was not to enforce on clients a single rigid data model but rather to provide a flexible foundation onto which clients can map their own data models.  The core entities in LUSID provide a minimal structure and set of relationships, and the data model can be extended using Properties.  The LUSID data model is exposed through the LUSID APIs.  The APIs provide access to both business objects and the meta data used to configure the systems behaviours.   The key business entities are: - * **Portfolios** A portfolio is a container for transactions and holdings (a **Transaction Portfolio**) or constituents (a **Reference Portfolio**). * **Derived Portfolios**. Derived Portfolios allow Portfolios to be created based on other Portfolios, by overriding or adding specific items. * **Holdings** A Holding is a quantity of an Instrument or a balance of cash within a Portfolio.  Holdings can only be adjusted via Transactions. * **Transactions** A Transaction is an economic event that occurs in a Portfolio, causing its holdings to change. * **Corporate Actions** A corporate action is a market event which occurs to an Instrument and thus applies to all portfolios which holding the instrument.  Examples are stock splits or mergers. * **Constituents** A constituent is a record in a Reference Portfolio containing an Instrument and an associated weight. * **Instruments**  An instrument represents a currency, tradable instrument or OTC contract that is attached to a transaction and a holding. * **Properties** All major entities allow additional user defined properties to be associated with them.   For example, a Portfolio manager may be associated with a portfolio.  Meta data includes: - * **Transaction Types** Transactions are booked with a specific transaction type.  The types are client defined and are used to map the Transaction to a series of movements which update the portfolio holdings.  * **Properties Types** Types of user defined properties used within the system.  ## Scope  All data in LUSID is segregated at the client level.  Entities in LUSID are identifiable by a unique code.  Every entity lives within a logical data partition known as a Scope.  Scope is an identity namespace allowing two entities with the same unique code to co-exist within individual address spaces.  For example, prices for equities from different vendors may be uploaded into different scopes such as `client/vendor1` and `client/vendor2`.  A portfolio may then be valued using either of the price sources by referencing the appropriate scope.  LUSID Clients cannot access scopes of other clients.  ## Instruments  LUSID has its own built-in instrument master which you can use to master your own instrument universe.  Every instrument must be created with one or more unique market identifiers, such as [FIGI](https://openfigi.com/). For any non-listed instruments (eg OTCs), you can upload an instrument against a custom ID of your choosing.  In addition, LUSID will allocate each instrument a unique 'LUSID instrument identifier'. The LUSID instrument identifier is what is used when uploading transactions, holdings, prices, etc. The API exposes an `instrument/lookup` endpoint which can be used to lookup these LUSID identifiers using their market identifiers.  Cash can be referenced using the ISO currency code prefixed with \"`CCY_`\" e.g. `CCY_GBP`  ## Instrument Data  Instrument data can be uploaded to the system using the [Instrument Properties](#tag/InstrumentProperties) endpoint.  | Field|Type|Description | | ---|---|--- | | Key|propertykey|The key of the property. This takes the format {domain}/{scope}/{code} e.g. 'Instrument/system/Name' or 'Transaction/strategy/quantsignal'. | | Value|string|The value of the property. | | EffectiveFrom|datetimeoffset|The effective datetime from which the property is valid. |   ## Transaction Portfolios  Portfolios are the top-level entity containers within LUSID, containing transactions, corporate actions and holdings.    The transactions build up the portfolio holdings on which valuations, analytics profit & loss and risk can be calculated.  Properties can be associated with Portfolios to add in additional data.  Portfolio properties can be changed over time, for example to allow a Portfolio Manager to be linked with a Portfolio.  Additionally, portfolios can be securitised and held by other portfolios, allowing LUSID to perform \"drill-through\" into underlying fund holdings  ### Derived Portfolios  LUSID also allows for a portfolio to be composed of another portfolio via derived portfolios.  A derived portfolio can contain its own transactions and also inherits any transactions from its parent portfolio.  Any changes made to the parent portfolio are automatically reflected in derived portfolio.  Derived portfolios in conjunction with scopes are a powerful construct.  For example, to do pre-trade what-if analysis, a derived portfolio could be created a new namespace linked to the underlying live (parent) portfolio.  Analysis can then be undertaken on the derived portfolio without affecting the live portfolio.  ### Transactions  A transaction represents an economic activity against a Portfolio.  Transactions are processed according to a configuration. This will tell the LUSID engine how to interpret the transaction and correctly update the holdings. LUSID comes with a set of transaction types you can use out of the box, or you can configure your own set(s) of transactions.  For more details see the [LUSID Getting Started Guide for transaction configuration.](https://support.lusid.com/configuring-transaction-types)  | Field|Type|Description | | ---|---|--- | | TransactionId|string|The unique identifier for the transaction. | | Type|string|The type of the transaction e.g. 'Buy', 'Sell'. The transaction type should have been pre-configured via the System Configuration API endpoint. If it hasn't been pre-configured the transaction will still be updated or inserted however you will be unable to generate the resultant holdings for the portfolio that contains this transaction as LUSID does not know how to process it. | | InstrumentIdentifiers|map|A set of instrument identifiers to use to resolve the transaction to a unique instrument. | | TransactionDate|dateorcutlabel|The date of the transaction. | | SettlementDate|dateorcutlabel|The settlement date of the transaction. | | Units|decimal|The number of units transacted in the associated instrument. | | TransactionPrice|transactionprice|The price for each unit of the transacted instrument in the transaction currency. | | TotalConsideration|currencyandamount|The total value of the transaction in the settlement currency. | | ExchangeRate|decimal|The exchange rate between the transaction and settlement currency. For example if the transaction currency is in USD and the settlement currency is in GBP this this the USD/GBP rate. | | TransactionCurrency|currency|The transaction currency. | | Properties|map|Set of unique transaction properties and associated values to store with the transaction. Each property must be from the 'Transaction' domain. | | CounterpartyId|string|The identifier for the counterparty of the transaction. | | Source|string|The source of the transaction. This is used to look up the appropriate transaction group set in the transaction type configuration. |   From these fields, the following values can be calculated  * **Transaction value in Transaction currency**: TotalConsideration / ExchangeRate  * **Transaction value in Portfolio currency**: Transaction value in Transaction currency * TradeToPortfolioRate  #### Example Transactions  ##### A Common Purchase Example Three example transactions are shown in the table below.   They represent a purchase of USD denominated IBM shares within a Sterling denominated portfolio.   * The first two transactions are for separate buy and fx trades    * Buying 500 IBM shares for $71,480.00    * A spot foreign exchange conversion to fund the IBM purchase. (Buy $71,480.00 for &#163;54,846.60)  * The third transaction is an alternate version of the above trades. Buying 500 IBM shares and settling directly in Sterling.  | Column |  Buy Trade | Fx Trade | Buy Trade with foreign Settlement | | ----- | ----- | ----- | ----- | | TransactionId | FBN00001 | FBN00002 | FBN00003 | | Type | Buy | FxBuy | Buy | | InstrumentIdentifiers | { \"figi\", \"BBG000BLNNH6\" } | { \"CCY\", \"CCY_USD\" } | { \"figi\", \"BBG000BLNNH6\" } | | TransactionDate | 2018-08-02 | 2018-08-02 | 2018-08-02 | | SettlementDate | 2018-08-06 | 2018-08-06 | 2018-08-06 | | Units | 500 | 71480 | 500 | | TransactionPrice | 142.96 | 1 | 142.96 | | TradeCurrency | USD | USD | USD | | ExchangeRate | 1 | 0.7673 | 0.7673 | | TotalConsideration.Amount | 71480.00 | 54846.60 | 54846.60 | | TotalConsideration.Currency | USD | GBP | GBP | | Trade/default/TradeToPortfolioRate&ast; | 0.7673 | 0.7673 | 0.7673 |  [&ast; This is a property field]  ##### A Forward FX Example  LUSID has a flexible transaction modelling system, meaning there are a number of different ways of modelling forward fx trades.  The default LUSID transaction types are FwdFxBuy and FwdFxSell. Using these transaction types, LUSID will generate two holdings for each Forward FX trade, one for each currency in the trade.  An example Forward Fx trade to sell GBP for USD in a JPY-denominated portfolio is shown below:  | Column | Forward 'Sell' Trade | Notes | | ----- | ----- | ---- | | TransactionId | FBN00004 | | | Type | FwdFxSell | | | InstrumentIdentifiers | { \"Instrument/default/Currency\", \"GBP\" } | | | TransactionDate | 2018-08-02 | | | SettlementDate | 2019-02-06 | Six month forward | | Units | 10000.00 | Units of GBP | | TransactionPrice | 1 | | | TradeCurrency | GBP | Currency being sold | | ExchangeRate | 1.3142 | Agreed rate between GBP and USD | | TotalConsideration.Amount | 13142.00 | Amount in the settlement currency, USD | | TotalConsideration.Currency | USD | Settlement currency | | Trade/default/TradeToPortfolioRate | 142.88 | Rate between trade currency, GBP and portfolio base currency, JPY |  Please note that exactly the same economic behaviour could be modelled using the FwdFxBuy Transaction Type with the amounts and rates reversed.  ### Holdings  A holding represents a position in an instrument or cash on a given date.  | Field|Type|Description | | ---|---|--- | | InstrumentUid|string|The unqiue Lusid Instrument Id (LUID) of the instrument that the holding is in. | | SubHoldingKeys|map|The sub-holding properties which identify the holding. Each property will be from the 'Transaction' domain. These are configured when a transaction portfolio is created. | | Properties|map|The properties which have been requested to be decorated onto the holding. These will be from the 'Instrument' or 'Holding' domain. | | HoldingType|string|The type of the holding e.g. Position, Balance, CashCommitment, Receivable, ForwardFX etc. | | Units|decimal|The total number of units of the holding. | | SettledUnits|decimal|The total number of settled units of the holding. | | Cost|currencyandamount|The total cost of the holding in the transaction currency. | | CostPortfolioCcy|currencyandamount|The total cost of the holding in the portfolio currency. | | Transaction|transaction|The transaction associated with an unsettled holding. |   ## Corporate Actions  Corporate actions are represented within LUSID in terms of a set of instrument-specific 'transitions'.  These transitions are used to specify the participants of the corporate action, and the effect that the corporate action will have on holdings in those participants.  ### Corporate Action  | Field|Type|Description | | ---|---|--- | | CorporateActionCode|code|The unique identifier of this corporate action | | Description|string|  | | AnnouncementDate|datetimeoffset|The announcement date of the corporate action | | ExDate|datetimeoffset|The ex date of the corporate action | | RecordDate|datetimeoffset|The record date of the corporate action | | PaymentDate|datetimeoffset|The payment date of the corporate action | | Transitions|corporateactiontransition[]|The transitions that result from this corporate action |   ### Transition | Field|Type|Description | | ---|---|--- | | InputTransition|corporateactiontransitioncomponent|Indicating the basis of the corporate action - which security and how many units | | OutputTransitions|corporateactiontransitioncomponent[]|What will be generated relative to the input transition |   ### Example Corporate Action Transitions  #### A Dividend Action Transition  In this example, for each share of IBM, 0.20 units (or 20 pence) of GBP are generated.  | Column |  Input Transition | Output Transition | | ----- | ----- | ----- | | Instrument Identifiers | { \"figi\" : \"BBG000BLNNH6\" } | { \"ccy\" : \"CCY_GBP\" } | | Units Factor | 1 | 0.20 | | Cost Factor | 1 | 0 |  #### A Split Action Transition  In this example, for each share of IBM, we end up with 2 units (2 shares) of IBM, with total value unchanged.  | Column |  Input Transition | Output Transition | | ----- | ----- | ----- | | Instrument Identifiers | { \"figi\" : \"BBG000BLNNH6\" } | { \"figi\" : \"BBG000BLNNH6\" } | | Units Factor | 1 | 2 | | Cost Factor | 1 | 1 |  #### A Spinoff Action Transition  In this example, for each share of IBM, we end up with 1 unit (1 share) of IBM and 3 units (3 shares) of Celestica, with 85% of the value remaining on the IBM share, and 5% in each Celestica share (15% total).  | Column |  Input Transition | Output Transition 1 | Output Transition 2 | | ----- | ----- | ----- | ----- | | Instrument Identifiers | { \"figi\" : \"BBG000BLNNH6\" } | { \"figi\" : \"BBG000BLNNH6\" } | { \"figi\" : \"BBG000HBGRF3\" } | | Units Factor | 1 | 1 | 3 | | Cost Factor | 1 | 0.85 | 0.15 |  ## Reference Portfolios Reference portfolios are portfolios that contain constituents with weights.  They are designed to represent entities such as indices and benchmarks.  ### Constituents | Field|Type|Description | | ---|---|--- | | InstrumentIdentifiers|map|Unique instrument identifiers | | InstrumentUid|string|LUSID's internal unique instrument identifier, resolved from the instrument identifiers | | Currency|decimal|  | | Weight|decimal|  | | FloatingWeight|decimal|  |   ## Portfolio Groups Portfolio groups allow the construction of a hierarchy from portfolios and groups.  Portfolio operations on the group are executed on an aggregated set of portfolios in the hierarchy.   For example:   * Global Portfolios _(group)_   * APAC _(group)_     * Hong Kong _(portfolio)_     * Japan _(portfolio)_   * Europe _(group)_     * France _(portfolio)_     * Germany _(portfolio)_   * UK _(portfolio)_   In this example **Global Portfolios** is a group that consists of an aggregate of **Hong Kong**, **Japan**, **France**, **Germany** and **UK** portfolios.  ## Properties  Properties are key-value pairs that can be applied to any entity within a domain (where a domain is `trade`, `portfolio`, `security` etc).  Properties must be defined before use with a `PropertyDefinition` and can then subsequently be added to entities.   ## Schema  A detailed description of the entities used by the API and parameters for endpoints which take a JSON document can be retrieved via the `schema` endpoint.  ## Meta data  The following headers are returned on all responses from LUSID  | Name | Purpose | | --- | --- | | lusid-meta-duration | Duration of the request | | lusid-meta-success | Whether or not LUSID considered the request to be successful | | lusid-meta-requestId | The unique identifier for the request | | lusid-schema-url | Url of the schema for the data being returned | | lusid-property-schema-url | Url of the schema for any properties |   # Error Codes  | Code|Name|Description | | ---|---|--- | | <a name=\"-10\">-10</a>|Server Configuration Error|  | | <a name=\"-1\">-1</a>|Unknown error|An unexpected error was encountered on our side. | | <a name=\"102\">102</a>|Version Not Found|  | | <a name=\"103\">103</a>|Api Rate Limit Violation|  | | <a name=\"104\">104</a>|Instrument Not Found|  | | <a name=\"105\">105</a>|Property Not Found|  | | <a name=\"106\">106</a>|Portfolio Recursion Depth|  | | <a name=\"108\">108</a>|Group Not Found|  | | <a name=\"109\">109</a>|Portfolio Not Found|  | | <a name=\"110\">110</a>|Property Schema Not Found|  | | <a name=\"111\">111</a>|Portfolio Ancestry Not Found|  | | <a name=\"112\">112</a>|Portfolio With Id Already Exists|  | | <a name=\"113\">113</a>|Orphaned Portfolio|  | | <a name=\"119\">119</a>|Missing Base Claims|  | | <a name=\"121\">121</a>|Property Not Defined|  | | <a name=\"122\">122</a>|Cannot Delete System Property|  | | <a name=\"123\">123</a>|Cannot Modify Immutable Property Field|  | | <a name=\"124\">124</a>|Property Already Exists|  | | <a name=\"125\">125</a>|Invalid Property Life Time|  | | <a name=\"126\">126</a>|Property Constraint Style Excludes Properties|  | | <a name=\"127\">127</a>|Cannot Modify Default Data Type|  | | <a name=\"128\">128</a>|Group Already Exists|  | | <a name=\"129\">129</a>|No Such Data Type|  | | <a name=\"130\">130</a>|Undefined Value For Data Type|  | | <a name=\"131\">131</a>|Unsupported Value Type Defined On Data Type|  | | <a name=\"132\">132</a>|Validation Error|  | | <a name=\"133\">133</a>|Loop Detected In Group Hierarchy|  | | <a name=\"134\">134</a>|Undefined Acceptable Values|  | | <a name=\"135\">135</a>|Sub Group Already Exists|  | | <a name=\"138\">138</a>|Price Source Not Found|  | | <a name=\"139\">139</a>|Analytic Store Not Found|  | | <a name=\"141\">141</a>|Analytic Store Already Exists|  | | <a name=\"143\">143</a>|Client Instrument Already Exists|  | | <a name=\"144\">144</a>|Duplicate In Parameter Set|  | | <a name=\"147\">147</a>|Results Not Found|  | | <a name=\"148\">148</a>|Order Field Not In Result Set|  | | <a name=\"149\">149</a>|Operation Failed|  | | <a name=\"150\">150</a>|Elastic Search Error|  | | <a name=\"151\">151</a>|Invalid Parameter Value|  | | <a name=\"153\">153</a>|Command Processing Failure|  | | <a name=\"154\">154</a>|Entity State Construction Failure|  | | <a name=\"155\">155</a>|Entity Timeline Does Not Exist|  | | <a name=\"156\">156</a>|Event Publish Failure|  | | <a name=\"157\">157</a>|Invalid Request|  | | <a name=\"158\">158</a>|Event Publish Unknown|  | | <a name=\"159\">159</a>|Event Query Failure|  | | <a name=\"160\">160</a>|Blob Did Not Exist|  | | <a name=\"162\">162</a>|Sub System Request Failure|  | | <a name=\"163\">163</a>|Sub System Configuration Failure|  | | <a name=\"165\">165</a>|Failed To Delete|  | | <a name=\"166\">166</a>|Upsert Client Instrument Failure|  | | <a name=\"167\">167</a>|Illegal As At Interval|  | | <a name=\"168\">168</a>|Illegal Bitemporal Query|  | | <a name=\"169\">169</a>|Invalid Alternate Id|  | | <a name=\"170\">170</a>|Cannot Add Source Portfolio Property Explicitly|  | | <a name=\"171\">171</a>|Entity Already Exists In Group|  | | <a name=\"173\">173</a>|Entity With Id Already Exists|  | | <a name=\"174\">174</a>|Derived Portfolio Details Do Not Exist|  | | <a name=\"176\">176</a>|Portfolio With Name Already Exists|  | | <a name=\"177\">177</a>|Invalid Transactions|  | | <a name=\"178\">178</a>|Reference Portfolio Not Found|  | | <a name=\"179\">179</a>|Duplicate Id|  | | <a name=\"180\">180</a>|Command Retrieval Failure|  | | <a name=\"181\">181</a>|Data Filter Application Failure|  | | <a name=\"182\">182</a>|Search Failed|  | | <a name=\"183\">183</a>|Movements Engine Configuration Key Failure|  | | <a name=\"184\">184</a>|Fx Rate Source Not Found|  | | <a name=\"185\">185</a>|Accrual Source Not Found|  | | <a name=\"186\">186</a>|Access Denied|  | | <a name=\"187\">187</a>|Invalid Identity Token|  | | <a name=\"188\">188</a>|Invalid Request Headers|  | | <a name=\"189\">189</a>|Price Not Found|  | | <a name=\"190\">190</a>|Invalid Sub Holding Keys Provided|  | | <a name=\"191\">191</a>|Duplicate Sub Holding Keys Provided|  | | <a name=\"192\">192</a>|Cut Definition Not Found|  | | <a name=\"193\">193</a>|Cut Definition Invalid|  | | <a name=\"194\">194</a>|Time Variant Property Deletion Date Unspecified|  | | <a name=\"195\">195</a>|Perpetual Property Deletion Date Specified|  | | <a name=\"196\">196</a>|Time Variant Property Upsert Date Unspecified|  | | <a name=\"197\">197</a>|Perpetual Property Upsert Date Specified|  | | <a name=\"200\">200</a>|Invalid Unit For Data Type|  | | <a name=\"201\">201</a>|Invalid Type For Data Type|  | | <a name=\"202\">202</a>|Invalid Value For Data Type|  | | <a name=\"203\">203</a>|Unit Not Defined For Data Type|  | | <a name=\"204\">204</a>|Units Not Supported On Data Type|  | | <a name=\"205\">205</a>|Cannot Specify Units On Data Type|  | | <a name=\"206\">206</a>|Unit Schema Inconsistent With Data Type|  | | <a name=\"207\">207</a>|Unit Definition Not Specified|  | | <a name=\"208\">208</a>|Duplicate Unit Definitions Specified|  | | <a name=\"209\">209</a>|Invalid Units Definition|  | | <a name=\"210\">210</a>|Invalid Instrument Identifier Unit|  | | <a name=\"211\">211</a>|Holdings Adjustment Does Not Exist|  | | <a name=\"212\">212</a>|Could Not Build Excel Url|  | | <a name=\"213\">213</a>|Could Not Get Excel Version|  | | <a name=\"214\">214</a>|Instrument By Code Not Found|  | | <a name=\"215\">215</a>|Entity Schema Does Not Exist|  | | <a name=\"216\">216</a>|Feature Not Supported On Portfolio Type|  | | <a name=\"217\">217</a>|Quote Not Found|  | | <a name=\"218\">218</a>|Invalid Quote Identifier|  | | <a name=\"219\">219</a>|Invalid Metric For Data Type|  | | <a name=\"220\">220</a>|Invalid Instrument Definition|  | | <a name=\"221\">221</a>|Instrument Upsert Failure|  | | <a name=\"222\">222</a>|Reference Portfolio Request Not Supported|  | | <a name=\"223\">223</a>|Transaction Portfolio Request Not Supported|  | | <a name=\"224\">224</a>|Invalid Property Value Assignment|  | | <a name=\"230\">230</a>|Transaction Type Not Found|  | | <a name=\"231\">231</a>|Transaction Type Duplication|  | | <a name=\"232\">232</a>|Portfolio Does Not Exist At Given Date|  | | <a name=\"233\">233</a>|Query Parser Failure|  | | <a name=\"234\">234</a>|Duplicate Constituent|  | | <a name=\"235\">235</a>|Unresolved Instrument Constituent|  | | <a name=\"236\">236</a>|Unresolved Instrument In Transition|  | | <a name=\"237\">237</a>|Missing Side Definitions|  | | <a name=\"300\">300</a>|Missing Recipe|  | | <a name=\"301\">301</a>|Dependencies|  | | <a name=\"304\">304</a>|Portfolio Preprocess Failure|  | | <a name=\"310\">310</a>|Valuation Engine Failure|  | | <a name=\"311\">311</a>|Task Factory Failure|  | | <a name=\"312\">312</a>|Task Evaluation Failure|  | | <a name=\"313\">313</a>|Task Generation Failure|  | | <a name=\"314\">314</a>|Engine Configuration Failure|  | | <a name=\"315\">315</a>|Model Specification Failure|  | | <a name=\"320\">320</a>|Market Data Key Failure|  | | <a name=\"321\">321</a>|Market Resolver Failure|  | | <a name=\"322\">322</a>|Market Data Failure|  | | <a name=\"330\">330</a>|Curve Failure|  | | <a name=\"331\">331</a>|Volatility Surface Failure|  | | <a name=\"332\">332</a>|Volatility Cube Failure|  | | <a name=\"350\">350</a>|Instrument Failure|  | | <a name=\"351\">351</a>|Cash Flows Failure|  | | <a name=\"352\">352</a>|Reference Data Failure|  | | <a name=\"360\">360</a>|Aggregation Failure|  | | <a name=\"361\">361</a>|Aggregation Measure Failure|  | | <a name=\"370\">370</a>|Result Retrieval Failure|  | | <a name=\"371\">371</a>|Result Processing Failure|  | | <a name=\"372\">372</a>|Vendor Result Processing Failure|  | | <a name=\"373\">373</a>|Vendor Result Mapping Failure|  | | <a name=\"374\">374</a>|Vendor Library Unauthorised|  | | <a name=\"375\">375</a>|Vendor Connectivity Error|  | | <a name=\"376\">376</a>|Vendor Interface Error|  | | <a name=\"377\">377</a>|Vendor Pricing Failure|  | | <a name=\"378\">378</a>|Vendor Translation Failure|  | | <a name=\"379\">379</a>|Vendor Key Mapping Failure|  | | <a name=\"380\">380</a>|Vendor Reflection Failure|  | | <a name=\"390\">390</a>|Attempt To Upsert Duplicate Quotes|  | | <a name=\"391\">391</a>|Corporate Action Source Does Not Exist|  | | <a name=\"392\">392</a>|Corporate Action Source Already Exists|  | | <a name=\"393\">393</a>|Instrument Identifier Already In Use|  | | <a name=\"394\">394</a>|Properties Not Found|  | | <a name=\"395\">395</a>|Batch Operation Aborted|  | | <a name=\"400\">400</a>|Invalid Iso4217 Currency Code|  | | <a name=\"401\">401</a>|Cannot Assign Instrument Identifier To Currency|  | | <a name=\"402\">402</a>|Cannot Assign Currency Identifier To Non Currency|  | | <a name=\"403\">403</a>|Currency Instrument Cannot Be Deleted|  | | <a name=\"404\">404</a>|Currency Instrument Cannot Have Economic Definition|  | | <a name=\"405\">405</a>|Currency Instrument Cannot Have Lookthrough Portfolio|  | | <a name=\"406\">406</a>|Cannot Create Currency Instrument With Multiple Identifiers|  | | <a name=\"407\">407</a>|Specified Currency Is Undefined|  | | <a name=\"410\">410</a>|Index Does Not Exist|  | | <a name=\"411\">411</a>|Sort Field Does Not Exist|  | | <a name=\"413\">413</a>|Negative Pagination Parameters|  | | <a name=\"414\">414</a>|Invalid Search Syntax|  | | <a name=\"420\">420</a>|Side Definition Inconsistent|  | | <a name=\"450\">450</a>|Invalid Quote Access Metadata Rule|  | | <a name=\"451\">451</a>|Access Metadata Not Found|  | | <a name=\"452\">452</a>|Invalid Access Metadata Identifier|  | | <a name=\"460\">460</a>|Standard Resource Not Found|  | | <a name=\"461\">461</a>|Standard Resource Conflict|  | | <a name=\"601\">601</a>|Person Identifier Already In Use|  | | <a name=\"602\">602</a>|Person Not Found|  | | <a name=\"603\">603</a>|Cannot Set Identifier|  | | <a name=\"617\">617</a>|Invalid Recipe Specification In Request|  | | <a name=\"618\">618</a>|Inline Recipe Deserialisation Failure|  | | <a name=\"619\">619</a>|Identifier Types Not Set For Entity Object|  | | <a name=\"620\">620</a>|Cannot Delete All Client Defined Identifiers|  | | <a name=\"650\">650</a>|The Order requested was not found.|  | | <a name=\"654\">654</a>|The Allocation requested was not found.|  | | <a name=\"655\">655</a>|Cannot build the fx forward target with the given holdings.|  | | <a name=\"656\">656</a>|Group does not contain expected entities.|  |   # noqa: E501

    The version of the OpenAPI document: 0.10.1400
    Contact: info@finbourne.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "0.10.1400"

# import apis into sdk package
from lusid.api.aggregation_api import AggregationApi
from lusid.api.allocations_api import AllocationsApi
from lusid.api.application_metadata_api import ApplicationMetadataApi
from lusid.api.configuration_recipe_api import ConfigurationRecipeApi
from lusid.api.corporate_action_sources_api import CorporateActionSourcesApi
from lusid.api.cut_label_definitions_api import CutLabelDefinitionsApi
from lusid.api.data_types_api import DataTypesApi
from lusid.api.derived_transaction_portfolios_api import DerivedTransactionPortfoliosApi
from lusid.api.entities_api import EntitiesApi
from lusid.api.instruments_api import InstrumentsApi
from lusid.api.login_api import LoginApi
from lusid.api.orders_api import OrdersApi
from lusid.api.persons_api import PersonsApi
from lusid.api.portfolio_groups_api import PortfolioGroupsApi
from lusid.api.portfolios_api import PortfoliosApi
from lusid.api.property_definitions_api import PropertyDefinitionsApi
from lusid.api.quotes_api import QuotesApi
from lusid.api.reconciliations_api import ReconciliationsApi
from lusid.api.reference_portfolio_api import ReferencePortfolioApi
from lusid.api.results_api import ResultsApi
from lusid.api.schemas_api import SchemasApi
from lusid.api.scopes_api import ScopesApi
from lusid.api.search_api import SearchApi
from lusid.api.structured_market_data_api import StructuredMarketDataApi
from lusid.api.structured_result_data_api import StructuredResultDataApi
from lusid.api.system_configuration_api import SystemConfigurationApi
from lusid.api.transaction_portfolios_api import TransactionPortfoliosApi

# import ApiClient
from lusid.api_client import ApiClient
from lusid.configuration import Configuration
from lusid.exceptions import OpenApiException
from lusid.exceptions import ApiTypeError
from lusid.exceptions import ApiValueError
from lusid.exceptions import ApiKeyError
from lusid.exceptions import ApiException
# import models into sdk package
from lusid.models.access_controlled_action import AccessControlledAction
from lusid.models.access_controlled_resource import AccessControlledResource
from lusid.models.access_metadata_value import AccessMetadataValue
from lusid.models.action_id import ActionId
from lusid.models.adjust_holding import AdjustHolding
from lusid.models.adjust_holding_request import AdjustHoldingRequest
from lusid.models.aggregate_spec import AggregateSpec
from lusid.models.aggregation_context import AggregationContext
from lusid.models.aggregation_measure_failure_detail import AggregationMeasureFailureDetail
from lusid.models.aggregation_options import AggregationOptions
from lusid.models.aggregation_request import AggregationRequest
from lusid.models.aggregation_response_node_of_dictionary_of_string_to_object import AggregationResponseNodeOfDictionaryOfStringToObject
from lusid.models.aggregation_result import AggregationResult
from lusid.models.allocation import Allocation
from lusid.models.allocation_request import AllocationRequest
from lusid.models.allocation_set_request import AllocationSetRequest
from lusid.models.annul_quotes_response import AnnulQuotesResponse
from lusid.models.annul_single_structured_data_response import AnnulSingleStructuredDataResponse
from lusid.models.annul_structured_data_response import AnnulStructuredDataResponse
from lusid.models.bond_instrument import BondInstrument
from lusid.models.bond_instrument_all_of import BondInstrumentAllOf
from lusid.models.cds_detail_specification import CdsDetailSpecification
from lusid.models.cds_instrument import CdsInstrument
from lusid.models.cds_instrument_all_of import CdsInstrumentAllOf
from lusid.models.change import Change
from lusid.models.complete_portfolio import CompletePortfolio
from lusid.models.configuration_recipe import ConfigurationRecipe
from lusid.models.configuration_recipe_snippet import ConfigurationRecipeSnippet
from lusid.models.constituents_adjustment_header import ConstituentsAdjustmentHeader
from lusid.models.corporate_action import CorporateAction
from lusid.models.corporate_action_source import CorporateActionSource
from lusid.models.corporate_action_transition import CorporateActionTransition
from lusid.models.corporate_action_transition_component import CorporateActionTransitionComponent
from lusid.models.corporate_action_transition_component_request import CorporateActionTransitionComponentRequest
from lusid.models.corporate_action_transition_request import CorporateActionTransitionRequest
from lusid.models.create_corporate_action_source_request import CreateCorporateActionSourceRequest
from lusid.models.create_cut_label_definition_request import CreateCutLabelDefinitionRequest
from lusid.models.create_data_type_request import CreateDataTypeRequest
from lusid.models.create_derived_transaction_portfolio_request import CreateDerivedTransactionPortfolioRequest
from lusid.models.create_portfolio_details import CreatePortfolioDetails
from lusid.models.create_portfolio_group_request import CreatePortfolioGroupRequest
from lusid.models.create_property_definition_request import CreatePropertyDefinitionRequest
from lusid.models.create_reference_portfolio_request import CreateReferencePortfolioRequest
from lusid.models.create_results import CreateResults
from lusid.models.create_transaction_portfolio_request import CreateTransactionPortfolioRequest
from lusid.models.create_unit_definition import CreateUnitDefinition
from lusid.models.currency_and_amount import CurrencyAndAmount
from lusid.models.cut_label_definition import CutLabelDefinition
from lusid.models.cut_local_time import CutLocalTime
from lusid.models.data_definition import DataDefinition
from lusid.models.data_mapping import DataMapping
from lusid.models.data_type import DataType
from lusid.models.delete_instrument_properties_response import DeleteInstrumentPropertiesResponse
from lusid.models.delete_instrument_response import DeleteInstrumentResponse
from lusid.models.deleted_entity_response import DeletedEntityResponse
from lusid.models.equity_option import EquityOption
from lusid.models.equity_option_all_of import EquityOptionAllOf
from lusid.models.error_detail import ErrorDetail
from lusid.models.execution_request import ExecutionRequest
from lusid.models.expanded_group import ExpandedGroup
from lusid.models.field_schema import FieldSchema
from lusid.models.file_response import FileResponse
from lusid.models.flow_conventions import FlowConventions
from lusid.models.fx_forward_instrument import FxForwardInstrument
from lusid.models.fx_forward_instrument_all_of import FxForwardInstrumentAllOf
from lusid.models.fx_option import FxOption
from lusid.models.fx_option_all_of import FxOptionAllOf
from lusid.models.get_instruments_response import GetInstrumentsResponse
from lusid.models.get_quotes_response import GetQuotesResponse
from lusid.models.get_recipe_response import GetRecipeResponse
from lusid.models.get_reference_portfolio_constituents_response import GetReferencePortfolioConstituentsResponse
from lusid.models.get_structured_market_data_response import GetStructuredMarketDataResponse
from lusid.models.get_structured_result_data_response import GetStructuredResultDataResponse
from lusid.models.holding_adjustment import HoldingAdjustment
from lusid.models.holdings_adjustment import HoldingsAdjustment
from lusid.models.holdings_adjustment_header import HoldingsAdjustmentHeader
from lusid.models.i_unit_definition_dto import IUnitDefinitionDto
from lusid.models.id_selector_definition import IdSelectorDefinition
from lusid.models.identifier_part_schema import IdentifierPartSchema
from lusid.models.inline_aggregation_request import InlineAggregationRequest
from lusid.models.inline_valuation_reconciliation_request import InlineValuationReconciliationRequest
from lusid.models.inline_valuations_reconciliation_request import InlineValuationsReconciliationRequest
from lusid.models.instrument import Instrument
from lusid.models.instrument_definition import InstrumentDefinition
from lusid.models.instrument_economic_definition import InstrumentEconomicDefinition
from lusid.models.instrument_id_type_descriptor import InstrumentIdTypeDescriptor
from lusid.models.instrument_id_value import InstrumentIdValue
from lusid.models.instrument_leg import InstrumentLeg
from lusid.models.instrument_match import InstrumentMatch
from lusid.models.instrument_search_property import InstrumentSearchProperty
from lusid.models.link import Link
from lusid.models.list_aggregation_response import ListAggregationResponse
from lusid.models.lusid_instrument import LusidInstrument
from lusid.models.lusid_instrument_definition import LusidInstrumentDefinition
from lusid.models.lusid_problem_details import LusidProblemDetails
from lusid.models.lusid_validation_problem_details import LusidValidationProblemDetails
from lusid.models.market_context import MarketContext
from lusid.models.market_context_suppliers import MarketContextSuppliers
from lusid.models.market_data_key_rule import MarketDataKeyRule
from lusid.models.market_data_manifest import MarketDataManifest
from lusid.models.market_options import MarketOptions
from lusid.models.metric_value import MetricValue
from lusid.models.model_property import ModelProperty
from lusid.models.model_selection import ModelSelection
from lusid.models.nested_aggregation_response import NestedAggregationResponse
from lusid.models.order import Order
from lusid.models.order_by_spec import OrderBySpec
from lusid.models.order_request import OrderRequest
from lusid.models.order_set_request import OrderSetRequest
from lusid.models.output_transaction import OutputTransaction
from lusid.models.paged_resource_list_of_allocation import PagedResourceListOfAllocation
from lusid.models.paged_resource_list_of_instrument import PagedResourceListOfInstrument
from lusid.models.paged_resource_list_of_order import PagedResourceListOfOrder
from lusid.models.paged_resource_list_of_portfolio_group import PagedResourceListOfPortfolioGroup
from lusid.models.paged_resource_list_of_portfolio_search_result import PagedResourceListOfPortfolioSearchResult
from lusid.models.paged_resource_list_of_property_definition import PagedResourceListOfPropertyDefinition
from lusid.models.perpetual_property import PerpetualProperty
from lusid.models.person import Person
from lusid.models.portfolio import Portfolio
from lusid.models.portfolio_details import PortfolioDetails
from lusid.models.portfolio_group import PortfolioGroup
from lusid.models.portfolio_group_properties import PortfolioGroupProperties
from lusid.models.portfolio_holding import PortfolioHolding
from lusid.models.portfolio_properties import PortfolioProperties
from lusid.models.portfolio_reconciliation_request import PortfolioReconciliationRequest
from lusid.models.portfolio_search_result import PortfolioSearchResult
from lusid.models.portfolios_reconciliation_request import PortfoliosReconciliationRequest
from lusid.models.portfolios_reconciliation_request_preview import PortfoliosReconciliationRequestPreview
from lusid.models.pricing_context import PricingContext
from lusid.models.pricing_options import PricingOptions
from lusid.models.processed_command import ProcessedCommand
from lusid.models.property_definition import PropertyDefinition
from lusid.models.property_filter import PropertyFilter
from lusid.models.property_schema import PropertySchema
from lusid.models.property_value import PropertyValue
from lusid.models.queryable_entities_response import QueryableEntitiesResponse
from lusid.models.quote import Quote
from lusid.models.quote_access_metadata_rule import QuoteAccessMetadataRule
from lusid.models.quote_access_metadata_rule_id import QuoteAccessMetadataRuleId
from lusid.models.quote_id import QuoteId
from lusid.models.quote_series_id import QuoteSeriesId
from lusid.models.realised_gain_loss import RealisedGainLoss
from lusid.models.reconciliation_break import ReconciliationBreak
from lusid.models.reference_portfolio_constituent import ReferencePortfolioConstituent
from lusid.models.reference_portfolio_constituent_request import ReferencePortfolioConstituentRequest
from lusid.models.resource_id import ResourceId
from lusid.models.resource_list_of_access_controlled_resource import ResourceListOfAccessControlledResource
from lusid.models.resource_list_of_access_metadata_value_of import ResourceListOfAccessMetadataValueOf
from lusid.models.resource_list_of_allocation import ResourceListOfAllocation
from lusid.models.resource_list_of_change import ResourceListOfChange
from lusid.models.resource_list_of_constituents_adjustment_header import ResourceListOfConstituentsAdjustmentHeader
from lusid.models.resource_list_of_corporate_action import ResourceListOfCorporateAction
from lusid.models.resource_list_of_corporate_action_source import ResourceListOfCorporateActionSource
from lusid.models.resource_list_of_cut_label_definition import ResourceListOfCutLabelDefinition
from lusid.models.resource_list_of_data_type import ResourceListOfDataType
from lusid.models.resource_list_of_get_recipe_response import ResourceListOfGetRecipeResponse
from lusid.models.resource_list_of_holdings_adjustment_header import ResourceListOfHoldingsAdjustmentHeader
from lusid.models.resource_list_of_i_unit_definition_dto import ResourceListOfIUnitDefinitionDto
from lusid.models.resource_list_of_instrument_id_type_descriptor import ResourceListOfInstrumentIdTypeDescriptor
from lusid.models.resource_list_of_order import ResourceListOfOrder
from lusid.models.resource_list_of_person import ResourceListOfPerson
from lusid.models.resource_list_of_portfolio import ResourceListOfPortfolio
from lusid.models.resource_list_of_portfolio_group import ResourceListOfPortfolioGroup
from lusid.models.resource_list_of_portfolio_search_result import ResourceListOfPortfolioSearchResult
from lusid.models.resource_list_of_processed_command import ResourceListOfProcessedCommand
from lusid.models.resource_list_of_property_definition import ResourceListOfPropertyDefinition
from lusid.models.resource_list_of_quote import ResourceListOfQuote
from lusid.models.resource_list_of_quote_access_metadata_rule import ResourceListOfQuoteAccessMetadataRule
from lusid.models.resource_list_of_reconciliation_break import ResourceListOfReconciliationBreak
from lusid.models.resource_list_of_scope_definition import ResourceListOfScopeDefinition
from lusid.models.resource_list_of_string import ResourceListOfString
from lusid.models.resource_list_of_value_type import ResourceListOfValueType
from lusid.models.result_data_key_rule import ResultDataKeyRule
from lusid.models.result_data_schema import ResultDataSchema
from lusid.models.results import Results
from lusid.models.schema import Schema
from lusid.models.scope_definition import ScopeDefinition
from lusid.models.side_configuration_data import SideConfigurationData
from lusid.models.side_configuration_data_request import SideConfigurationDataRequest
from lusid.models.stream import Stream
from lusid.models.structured_market_data import StructuredMarketData
from lusid.models.structured_market_data_id import StructuredMarketDataId
from lusid.models.structured_result_data import StructuredResultData
from lusid.models.structured_result_data_id import StructuredResultDataId
from lusid.models.swap_instrument import SwapInstrument
from lusid.models.swap_instrument_all_of import SwapInstrumentAllOf
from lusid.models.swaption import Swaption
from lusid.models.swaption_all_of import SwaptionAllOf
from lusid.models.target_tax_lot import TargetTaxLot
from lusid.models.target_tax_lot_request import TargetTaxLotRequest
from lusid.models.tenor import Tenor
from lusid.models.tolerance import Tolerance
from lusid.models.transaction import Transaction
from lusid.models.transaction_configuration_data import TransactionConfigurationData
from lusid.models.transaction_configuration_data_request import TransactionConfigurationDataRequest
from lusid.models.transaction_configuration_movement_data import TransactionConfigurationMovementData
from lusid.models.transaction_configuration_movement_data_request import TransactionConfigurationMovementDataRequest
from lusid.models.transaction_configuration_type_alias import TransactionConfigurationTypeAlias
from lusid.models.transaction_price import TransactionPrice
from lusid.models.transaction_property_mapping import TransactionPropertyMapping
from lusid.models.transaction_property_mapping_request import TransactionPropertyMappingRequest
from lusid.models.transaction_query_parameters import TransactionQueryParameters
from lusid.models.transaction_request import TransactionRequest
from lusid.models.transaction_set_configuration_data import TransactionSetConfigurationData
from lusid.models.transaction_set_configuration_data_request import TransactionSetConfigurationDataRequest
from lusid.models.update_cut_label_definition_request import UpdateCutLabelDefinitionRequest
from lusid.models.update_data_type_request import UpdateDataTypeRequest
from lusid.models.update_instrument_identifier_request import UpdateInstrumentIdentifierRequest
from lusid.models.update_portfolio_group_request import UpdatePortfolioGroupRequest
from lusid.models.update_portfolio_request import UpdatePortfolioRequest
from lusid.models.update_property_definition_request import UpdatePropertyDefinitionRequest
from lusid.models.upsert_corporate_action_request import UpsertCorporateActionRequest
from lusid.models.upsert_corporate_actions_response import UpsertCorporateActionsResponse
from lusid.models.upsert_instrument_properties_response import UpsertInstrumentPropertiesResponse
from lusid.models.upsert_instrument_property_request import UpsertInstrumentPropertyRequest
from lusid.models.upsert_instruments_response import UpsertInstrumentsResponse
from lusid.models.upsert_order_properties_request import UpsertOrderPropertiesRequest
from lusid.models.upsert_order_properties_response import UpsertOrderPropertiesResponse
from lusid.models.upsert_person_access_metadata_request import UpsertPersonAccessMetadataRequest
from lusid.models.upsert_person_request import UpsertPersonRequest
from lusid.models.upsert_portfolio_access_metadata_request import UpsertPortfolioAccessMetadataRequest
from lusid.models.upsert_portfolio_executions_response import UpsertPortfolioExecutionsResponse
from lusid.models.upsert_portfolio_group_access_metadata_request import UpsertPortfolioGroupAccessMetadataRequest
from lusid.models.upsert_portfolio_transactions_response import UpsertPortfolioTransactionsResponse
from lusid.models.upsert_quote_access_metadata_rule_request import UpsertQuoteAccessMetadataRuleRequest
from lusid.models.upsert_quote_request import UpsertQuoteRequest
from lusid.models.upsert_quotes_response import UpsertQuotesResponse
from lusid.models.upsert_recipe_request import UpsertRecipeRequest
from lusid.models.upsert_reference_portfolio_constituents_request import UpsertReferencePortfolioConstituentsRequest
from lusid.models.upsert_reference_portfolio_constituents_response import UpsertReferencePortfolioConstituentsResponse
from lusid.models.upsert_single_structured_data_response import UpsertSingleStructuredDataResponse
from lusid.models.upsert_structured_data_response import UpsertStructuredDataResponse
from lusid.models.upsert_structured_market_data_request import UpsertStructuredMarketDataRequest
from lusid.models.upsert_structured_result_data_request import UpsertStructuredResultDataRequest
from lusid.models.upsert_transaction_properties_response import UpsertTransactionPropertiesResponse
from lusid.models.user import User
from lusid.models.valuation_reconciliation_request import ValuationReconciliationRequest
from lusid.models.valuations_reconciliation_request import ValuationsReconciliationRequest
from lusid.models.value_tuple_of_market_data_key_rule_to_quote import ValueTupleOfMarketDataKeyRuleToQuote
from lusid.models.value_tuple_of_market_data_key_rule_to_structured_market_data import ValueTupleOfMarketDataKeyRuleToStructuredMarketData
from lusid.models.vendor_model_rule import VendorModelRule
from lusid.models.version import Version
from lusid.models.version_summary_dto import VersionSummaryDto
from lusid.models.versioned_resource_list_of_output_transaction import VersionedResourceListOfOutputTransaction
from lusid.models.versioned_resource_list_of_portfolio_holding import VersionedResourceListOfPortfolioHolding
from lusid.models.versioned_resource_list_of_transaction import VersionedResourceListOfTransaction
from lusid.models.weighted_instrument import WeightedInstrument

# import utilities into sdk package
from lusid.utilities.api_client_builder import ApiClientBuilder
from lusid.utilities.api_configuration import ApiConfiguration
from lusid.utilities.api_configuration_loader import ApiConfigurationLoader
from lusid.utilities.refreshing_token import RefreshingToken

# import tcp utilities
from lusid.tcp.tcp_keep_alive_probes import TCPKeepAlivePoolManager, TCPKeepAliveProxyManager
